"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.pageObjectsFromS3 = exports.listObjectsFromS3 = exports.split = exports.splitS3Object = exports.getObjectFromS3AsStream = exports.getObjectFromS3 = exports.toGetObjectRequest2 = exports.toGetObjectRequest = exports.deleteObjectFromS3 = exports.putObjectToS3 = void 0;

var _highland = _interopRequireDefault(require("highland"));

var _s = _interopRequireDefault(require("../connectors/s3"));

var _faults = require("./faults");

var _print = require("./print");

var _ratelimit = require("./ratelimit");

const putObjectToS3 = ({
  debug = (0, _print.debug)('s3'),
  bucketName = process.env.BUCKET_NAME,
  putRequestField = 'putRequest',
  putResponseField = 'putResponse',
  parallel = Number(process.env.S3_PARALLEL) || Number(process.env.PARALLEL) || 8,
  ...opt
} = {}) => {
  const connector = new _s.default({
    debug,
    bucketName
  });

  const putObject = uow => {
    if (!uow[putRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = connector.putObject(uow[putRequestField]).then(putResponse => ({ ...uow,
      [putResponseField]: putResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.through((0, _ratelimit.ratelimit)(opt)).map(putObject).parallel(parallel);
};

exports.putObjectToS3 = putObjectToS3;

const deleteObjectFromS3 = ({
  debug,
  bucketName = process.env.BUCKET_NAME,
  deleteRequestField = 'deleteRequest',
  deleteResponseField = 'deleteResponse',
  parallel = Number(process.env.S3_PARALLEL) || Number(process.env.PARALLEL) || 8
} = {}) => {
  const connector = new _s.default({
    debug,
    bucketName
  });

  const deleteObject = uow => {
    if (!uow[deleteRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = connector.deleteObject(uow[deleteRequestField]).then(deleteResponse => ({ ...uow,
      [deleteResponseField]: deleteResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.map(deleteObject).parallel(parallel);
};

exports.deleteObjectFromS3 = deleteObjectFromS3;

const toGetObjectRequest = uow => ({ ...uow,
  getRequest: {
    Bucket: uow.record.s3.bucket.name,
    Key: uow.record.s3.object.key
  }
});

exports.toGetObjectRequest = toGetObjectRequest;

const toGetObjectRequest2 = uow => ({ ...uow,
  getRequest: {
    Bucket: uow.record.s3.s3.bucket.name,
    Key: uow.record.s3.s3.object.key
  }
});

exports.toGetObjectRequest2 = toGetObjectRequest2;

const getObjectFromS3 = ({
  debug = (0, _print.debug)('s3'),
  bucketName = process.env.BUCKET_NAME,
  getRequestField = 'getRequest',
  getResponseField = 'getResponse',
  parallel = Number(process.env.S3_PARALLEL) || Number(process.env.PARALLEL) || 8
} = {}) => {
  const connector = new _s.default({
    debug,
    bucketName
  });

  const getObject = uow => {
    if (!uow[getRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = connector.getObject(uow[getRequestField]).then(getResponse => ({ ...uow,
      [getResponseField]: getResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.map(getObject).parallel(parallel);
};

exports.getObjectFromS3 = getObjectFromS3;

const getObjectFromS3AsStream = ({
  debug = (0, _print.debug)('s3'),
  bucketName = process.env.BUCKET_NAME,
  getRequestField = 'getRequest',
  getResponseField = 'getResponse',
  delimiter = '\n',
  splitFilter = () => true
} = {}) => {
  const connector = new _s.default({
    debug,
    bucketName
  });

  const getObject = uow => {
    if (!uow[getRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = connector.getObjectStream(uow[getRequestField]);
    return (0, _highland.default)(p) // wrap stream in a stream
    .splitBy(delimiter).filter(splitFilter).map(getResponse => ({ ...uow,
      [getResponseField]: getResponse
    }));
  };

  return s => s.flatMap(getObject);
};

exports.getObjectFromS3AsStream = getObjectFromS3AsStream;

const splitS3Object = ({
  delimiter = '\n',
  getResponseField = 'getResponse'
} = {}) => uow => {
  const {
    Body,
    ...rest
  } = uow[getResponseField];
  return (0, _highland.default)(Buffer.from(Body).toString().split(delimiter).filter(line => line.length !== 0).map(line => ({ ...uow,
    [getResponseField]: { ...rest,
      line
    }
  })));
};

exports.splitS3Object = splitS3Object;
const split = splitS3Object; // deprecated export

exports.split = split;

const listObjectsFromS3 = ({
  debug = (0, _print.debug)('s3'),
  bucketName = process.env.BUCKET_NAME,
  listRequestField = 'listRequest',
  listResponseField = 'listResponse',
  parallel = Number(process.env.S3_PARALLEL) || Number(process.env.PARALLEL) || 8
} = {}) => {
  const connector = new _s.default({
    debug,
    bucketName
  });

  const listObjects = uow => {
    /* istanbul ignore if */
    if (!uow[listRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = connector.listObjects(uow[listRequestField]).then(listResponse => ({ ...uow,
      [listResponseField]: listResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.map(listObjects).parallel(parallel);
};

exports.listObjectsFromS3 = listObjectsFromS3;

const pageObjectsFromS3 = ({
  debug = (0, _print.debug)('s3'),
  bucketName = process.env.BUCKET_NAME,
  listRequestField = 'listRequest',
  parallel = Number(process.env.S3_PARALLEL) || Number(process.env.PARALLEL) || 8
} = {}) => {
  const connector = new _s.default({
    debug,
    bucketName
  });

  const listObjects = uow => {
    let {
      ContinuationToken
    } = uow[listRequestField];
    return (0, _highland.default)((push, next) => {
      const params = { ...uow[listRequestField],
        ContinuationToken
      };
      connector.listObjects(params).then(data => {
        const {
          Contents,
          ...rest
        } = data;
        debug('listObjects: %j', rest);

        if (rest.IsTruncated) {
          ContinuationToken = rest.NextContinuationToken;
        } else {
          ContinuationToken = undefined;
        }

        Contents.forEach(obj => {
          push(null, { ...uow,
            [listRequestField]: params,
            listResponse: { ...rest,
              Content: obj
            }
          });
        });
      }).catch(
      /* istanbul ignore next */
      err => {
        err.uow = uow;
        push(err, null);
      }).finally(() => {
        if (ContinuationToken) {
          next();
        } else {
          push(null, _highland.default.nil);
        }
      });
    });
  };

  return s => s.map(listObjects).parallel(parallel);
};

exports.pageObjectsFromS3 = pageObjectsFromS3;